<?php

/**
 * Hash-based pseudo random number generator.
 *
 * Generates repeatable sequences of random numbers, using a beginning phrase as 
 * the initial hash.
 */
class RepeatablePseudoRNG
{
   /**
    * Binary string that contains the current sha1 round.
    */
   public $current_seed;

   public function __construct($seed = NULL)
   {
      if($seed === NULL)
      {
         $seed = time();
      }
      $this->current_seed = sha1($seed, true);
   }
   
   /**
    * Updates the seed.
    */
   public function update_seed()
   {
      $this->current_seed = sha1($this->current_seed, true);
   }
   
   /**
    * Update the seed an arbitary number of times.
    *
    * This is a simple convenience function to call update_seed multiple times.
    * @param Integer $number_of_rounds Number of times to to update the seed.
    */
   public function perform_rounds($number_of_rounds)
   {
      for($i < 0; $i < $number_of_rounds; $i++)
      {
         $this->update_seed();
      }
   }
   
   /**
    * Fetch current seed.
    * @return String hexadecimal string of current seed round.
    */
   public function fetch_seed()
   {
      return bin2hex($this->current_seed);
   }
   
   /**
    * Get an int from the current seed, and update the seed.
    *
    * Maximum range allowed is 2147483647. Max also cannot exceed PHP_INT_MAX.
    * @param Integer $min Minimum number
    * @param Integer $max Maximum number (no greater than 2147483647 of min.)
    * @return Int
    */
   public function fetch_int($min = 0, $max = 65535)
   {
      if($min == $max)
      {
         $this->update_seed();
         return $min;
      }
      
      if($max > PHP_INT_MAX)
      {
         $max = PHP_INT_MAX;
      }

      $num = 0;
      $data = unpack('V1/V2', $this->current_seed);
      $data[1] = $data[1]<0?$data[1]*-1:$data[1];
      $data[2] = $data[2]<0?$data[2]*-1:$data[2];
      $rangemask = $data[1] ^ $data[2];
      $num = $min + ($rangemask%($max-$min));
      
      $this->update_seed();
      return $num;
   }
   
   /**
    * Fetch floating point number.
    *
    * @param Float $min Minimum number
    * @param Float $max Maximum number
    * @return Float
    */
   public function fetch_float($min = -65535.0, $max = 65535.0)
   {
      if($min == $max)
      {
         $this->update_seed();
         return $min;
      }
      $num = 0;

      $num = $min + ($max-$min)*($this->rand());
      return $num;
   }
   
   /**
    * Generates a random number between 0 and 1.
    */
   public function rand()
   {
      $data = unpack('V1/V2', $this->current_seed);
      $data[1] = $data[1]<0?$data[1]*-1:$data[1];
      $data[2] = $data[2]<0?$data[2]*-1:$data[2];
      $rangemask = $data[1] ^ $data[2];
      $this->update_seed();
      
      if($rangemask % 1000000 == 0)
      {
         return 0;
      }
      else
      {
         if($rangemask%1000000 == 999999) return 1.0;
         return (floatval($rangemask%1000000)-0.5+(floatval($rangemask%200)/199))/999999;
      }
   }
}